export declare abstract class Flat<T> {
    abstract decode(decoder: DecoderState): T;
}
declare class Natural {
    n: bigint;
    constructor(n: bigint);
}
export declare class FlatNatural extends Flat<Natural> {
    private static readonly instance;
    static getInstance(): FlatNatural;
    decode(decoder: DecoderState): Natural;
}
export declare class FlatBigInt extends Flat<bigint> {
    private static readonly instance;
    static getInstance(): FlatBigInt;
    decode(decoder: DecoderState): bigint;
}
export declare class FlatArrayByte extends Flat<Int8Array> {
    private static readonly instance;
    static getInstance(): FlatArrayByte;
    decode(decoder: DecoderState): Int8Array;
}
export declare class FlatString extends Flat<string> {
    private static readonly instance;
    static getInstance(): FlatString;
    decode(decoder: DecoderState): string;
}
export declare class FlatBoolean extends Flat<boolean> {
    private static readonly instance;
    static getInstance(): FlatBoolean;
    decode(decoder: DecoderState): boolean;
}
export declare class FlatUnit extends Flat<string> {
    private static readonly instance;
    static getInstance(): FlatUnit;
    decode(decoder: DecoderState): string;
}
export declare class FlatData extends Flat<Int8Array> {
    private static readonly instance;
    static getInstance(): FlatData;
    decode(decoder: DecoderState): Int8Array;
}
export declare const zagZig: (u: bigint) => bigint;
export declare class DecoderState {
    /** The buffer that contains a sequence of flat-encoded values */
    buffer: Int8Array;
    /** Pointer to the current byte being decoded (0..buffer.byteLength-1) */
    currPtr: number;
    /** Number of already decoded bits in the current byte (0..7) */
    usedBits: number;
    constructor(buffer: Int8Array);
    toString(): string;
    /** Decode up to 8 bits
     * @param numBits
     *   the number of bits to decode (0..8)
     */
    bits8(numBits: number): number;
    filler(): void;
    private ensureBits;
    private availableBits;
    private availableBytes;
    private dropBits;
}
export declare class ListFlat<T> {
    decoder: Flat<T>;
    constructor(flat: Flat<T>);
    decode(decode: DecoderState): T[];
}
export {};
