import { Constant } from './utils';
import { DecoderState } from '../flat/flat';
export declare const fromDeBruijnTerm: (term: Term) => Term;
export declare class NamedDeBruijn {
    name: string;
    index: number;
    constructor(name: string, index?: number);
    toString(): string;
}
export declare enum TermType {
    Var = 0,
    LamAbs = 1,
    Apply = 2,
    Force = 3,
    Delay = 4,
    Const = 5,
    Builtin = 6,
    Error = 7
}
export declare class Term {
    type?: TermType;
    name?: NamedDeBruijn;
    term?: Term;
    f?: Term;
    arg?: Term;
    const?: Constant;
    bn?: string;
    static Var(name: NamedDeBruijn): Term;
    static LamAbs(name: string, term: Term): Term;
    static Apply(f: Term, arg: Term): Term;
    static Force(term: Term): Term;
    static Delay(term: Term): Term;
    static Const(constant: Constant): Term;
    static Builtin(defaultFun: string): Term;
    static Error(): Term;
    pretty(): string | undefined;
}
export type Version = {
    major: number;
    minor: number;
    patch: number;
};
export declare class DeBruijnedProgram {
    version: Version;
    term: Term;
    constructor(version: Version, term: Term);
    pretty(): string;
}
export declare const termTagWidth = 4;
export declare class FlatDeBruijnedProgram {
    static decode(decoder: DecoderState): DeBruijnedProgram;
}
export declare class FlatTerm {
    static decode(decoder: DecoderState): Term;
}
